/* diodemail - send-only smtp server
 * Copyright (c) 2024 Gnarwhal
 *
 * This file is part of SSHare.
 *
 * SSHare is free software: you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software Foundation,
 * either version 3 of the License, or (at your option) any later version.
 *
 * SSHare is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * SSHare. If not, see <https://www.gnu.org/licenses/>.
 */

package smtp

import (
	"fmt"
	"net"
	"crypto/tls"
	"sync"

	"github.com/rs/zerolog/log"
)

type PlainConfig struct {
	Port string
}

type TLSConfig struct {
	Port      string
	TlsConfig tls.Config
}

func handle(connection net.Conn, host string, password_hash string) {
	log.Info().Msgf(
		"New connection %v. Starting session.",
		connection.RemoteAddr(),
	)
	defer connection.Close()

	session := MakeSMTPSession(connection, host, password_hash)
	err := session.Run()
	if err != nil {
		log.Error().Msgf(
			"Session %v exited with error: %v",
			connection.RemoteAddr(),
			err,
		)
	} else {
		log.Info().Msgf(
			"Session %v exited successfully",
			connection.RemoteAddr(),
		)
	}
}

func Run(host string, password_hash string, plain_config *PlainConfig, tls_config *TLSConfig) error {
	var wait_group sync.WaitGroup
	if plain_config != nil {
		listener, err := net.Listen("tcp", fmt.Sprintf(":%v", plain_config.Port))
		if err != nil {
			return err
		}
		log.Info().Msgf("Plain text server started on port %v for host %v", plain_config.Port, host)
		wait_group.Add(1)
		go Listen(wait_group, host, password_hash, listener)
	}
	if tls_config != nil {
		listener, err := tls.Listen("tcp", fmt.Sprintf(":%v", tls_config.Port), &tls_config.TlsConfig)
		if err != nil {
			return err
		}
		log.Info().Msgf("TLS server started on port %v for host %v", tls_config.Port, host)
		wait_group.Add(1)
		go Listen(wait_group, host, password_hash, listener)
	}
	wait_group.Wait()

	return nil
}

func Listen(wait_group sync.WaitGroup, host string, password_hash string, listener net.Listener) {
	defer wait_group.Done()
	for {
		connection, _ := listener.Accept()

		go handle(connection, host, password_hash)
	}
}
